@@ -4,6 +4,7 @@ Model Management and simulation objects.
 """
 import cPickle, time
 import os, codecs
+from copy import deepcopy
 from simobj import graph, edge, siteobj
 from numpy import *
 from numpy.random import uniform
@@ -13,9 +14,12 @@ from Epigrass.data_io import *
 import ConfigParser
 import string, sys, getpass
 import Epigrass.report as Rp
+import epigdal 
 import encodings.utf_8
 import encodings.latin_1
 import dataObject as DO
+import sqlite3, MySQLdb
+#import pycallgraph
 
 # Import Psyco if available
 ##try:
@@ -44,7 +48,7 @@ class simulate:
             self.fname = fname
         else:
             self.fname = sys.argv[1]
-        
+        #pycallgraph.start_trace()
         self.host = host
         self.port = port
         self.usr = user
@@ -52,13 +56,21 @@ class simulate:
         self.backend = backend
         self.passw = password
         self.repname = None
+        self.shapefile = None
+        self.World = None
         self.dir = os.getcwd()
         sys.path.insert(0,self.dir) #add current path to sys.path
         self.gui = 0 # True if this object was created by the gui
         self.now = time.asctime().replace(' ','_').replace(':','') #current date and time
-        self.modelName = self.fname.split('.')[0]
+        self.modelName = os.path.split(self.fname)[-1].split('.')[0]
         self.config = self.loadModelScript(self.fname)
         self.evalConfig(self.config)
+        if not self.outdir:
+            self.outdir = outdir = 'outdata-'+self.modelName
+        if not os.path.exists(self.outdir): 
+            os.mkdir(self.outdir)
+        if self.shapefile: #create world object if a shapefile is provided
+            self.World = epigdal.World(*self.shapefile+[self.outdir])
         self.chkScript()
         self.encoding = 'latin-1'#default encoding for strings in the sites and edges files
         self.round = 0 #No replicates
@@ -96,7 +108,10 @@ class simulate:
         """
         try:
             #WORLD
-            #self.location = config['the world.location']
+            #load Shapefile if specified
+            if 'the world.shapefile' in config:
+                self.shapefile = eval(config['the world.shapefile'])
+                
             #self.layer = config['the world.vector layer']
             self.sites = config['the world.sites'] #file containing site info
             self.edges = config['the world.edges'] #file containing edge info
@@ -213,7 +228,7 @@ class simulate:
                 inits[k] = eval(v)
             for k,v in self.parms.items():
                 parms[k] = eval(v)
-            #Linhas abaixo estão obsoletas (remover qundo tiver tempo para testar)
+            #Linhas abaixo estao obsoletas (remover qundo tiver tempo para testar)
             I = eval(self.I)
             E = eval(self.E)
             S = eval(self.S)
@@ -241,6 +256,9 @@ class simulate:
              
             if self.seed:
                 #print type(o.geocode), type(self.seed[0])
+                if self.seed[0][0] == "all":
+                    I += 1
+                    inits[self.seed[0][1].lower()] += 1
                 for j in self.seed:
                     if int(o.geocode) == j[0]:
                         self.Say("%s infected case(s) arrived at %s"%(j[2],o.sitename))
@@ -268,6 +286,9 @@ class simulate:
         source = dest = None
         for edg in edgelist:
             #print edgelist.index(edg)
+            if edg[5] == edg[6]:
+                #ignore circular edges
+                continue
             for site in sitelist:
                 #print site.sitename
                 if str(site.geocode) == edg[5]: 
@@ -276,10 +297,10 @@ class simulate:
                 elif str(site.geocode) == edg[6]: 
                     dest = site
                     #print 'found dest = ',dest.sitename
-                if source and dest:
+                if (source and dest):
                     #print 'next edge!'
                     break
-            if not source and dest:
+            if not (source and dest):
                 print source ,dest
                 sys.exit('One of the vertices on edge '+edg[0]+'-'+edg[1]+' could not be found on the site list')
             
@@ -343,6 +364,7 @@ class simulate:
         """
         Start the simulation
         """
+        
         if '/' in self.modelName:
             self.modelName = os.path.split(self.modelName)[-1]
         start = time.time()
@@ -352,11 +374,15 @@ class simulate:
         if self.gui:
             self.gui.textEdit1.insertParagraph('Simulation lasted %s seconds.'%elapsed,-1)
         if self.MySQLout:
-            self.outToODb(self.modelName)
-            #self.outToDb(modelname)
-            #qnplot(self.outtable)
-        self.dumpData()
+            if self.backend == 'csv':
+                self.outToCsv(self.modelName)
+            else:
+                self.outToDb(self.modelName)
             
+        self.outToShp()
+        #self.dumpData()
+        #self.saveModel(self.modelName)
+        #pycallgraph.make_dot_graph(self.modelName+'_callgraph.png')
         #print len(self.g.site_list[-1].ts)
         
 ##        for i in self.g.site_list:
@@ -385,10 +411,104 @@ class simulate:
     def grassVect2ascii(self,layer):
         """
         Convert Grass vector layer to an ascii file epigrass can read
+        Deprecated!
         """
         com = 'v.out.ascii input=%s output=map.txt format=standard'%layer 
         os.system(com)
     
+    def outToShp(self):
+        """
+        Creates Data Layers in shapefile format
+        """
+        if not self.World:
+            return
+        #Initialize world output layers
+        self.Say("REading Nodes from shapefile...")
+        self.World.getNodeList(self.World.ds.GetLayerByName(self.World.layerlist[0]))
+        self.Say("Done reading nodes!")
+        self.Say("Creating Nodes shapefile...")
+        self.World.createNodeLayer()
+        self.Say("Done creating Nodes shapefile!")
+        elist = [(e.source.geocode,e.dest.geocode,sum(e.ftheta),sum(e.btheta)) for e in self.g.edge_list]
+        self.Say("Creating Edges shapefile...")
+        self.World.createEdgeLayer(elist)
+        self.Say("Done creating Edges shapefile!")
+        #Generate site epidemic stats
+        varlist = ["prevalence","totalcases", "arrivals","population"]
+        sitestats = []
+        names = {}
+        for site in self.g.site_list:
+            names[int(site.geocode)] = site.sitename
+            prevalence = float(site.totalcases)/site.totpop
+            #print prevalence
+            sitestats.append((site.geocode,prevalence,site.totalcases,sum(site.thetahist),float(site.totpop)))
+        
+        self.Say("Creating Data shapefile...")
+        self.World.createDataLayer(varlist,sitestats)
+        self.Say("Done creating Data shapefile!")
+        #Generate the kml too.
+        lr = self.World.datasource.GetLayer(0)
+        k = epigdal.KmlGenerator()
+        k.addNodes(lr,names)
+        k.writeToFile(self.outdir)
+        #close files
+        self.World.closeSources()
+
+        
+    def outToCsv(self,table):
+        """
+        Save simulation results in csv file.
+        """
+        tablee = table+"_"+self.now+"_e.tab" #edgetable name
+        table += "_"+self.now+".tab"#sitetable name
+        f = open(table,"w")
+        head = ['geocode','time','totpop','name',
+        'lat','longit']
+        headerwritten = False
+        for site in self.g.site_list:
+            t = 0
+            regb = [str(site.geocode),str(t),str(site.totpop),
+                site.sitename.replace('"','').encode('ascii','replace'),
+                str(site.pos[0]),str(site.pos[1])
+                ]
+            if site.values:
+                for n,v in enumerate(site.values):
+                    head.append('values%s'%n)
+                    regb.append(str(v))
+            
+            ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
+            head.extend(['incidence','arrivals'])
+            for n,v in enumerate(site.vnames):
+                    head.append(str(v))
+            for i in ts:
+                reg = deepcopy(regb)
+                reg.extend([str(site.incidence[t]),str(site.thetahist[t])])
+                reg[1] = str(t)
+                for n,v in enumerate(site.vnames):
+                    reg.append(str(i[n]))
+                if not headerwritten:
+                    h =",".join(head)
+                    f.write(h+'\n')
+                    headerwritten = True
+                f.write(','.join(reg)+'\n')
+                t += 1
+        f.close()
+        #inserting data in edges file
+        g = open(tablee,'w')
+        t=0
+        head = ['source_code','dest_code','time','ftheta','btheta']
+        ehw = False
+        for e in self.g.edge_list:
+            for f,b in zip(e.ftheta,e.btheta):
+                if not ehw:
+                    g.write(','.join(head)+'\n')
+                    ehw = True
+                ereg = [e.source.geocode,e.dest.geocode,t,f,b]
+                ereg = [str(i) for i in ereg]
+                g.write(','.join(ereg)+'\n')
+                t += 1
+        g.close()
+        
     def outToODb(self,table,mode='b'):
         """
         Insert simulation results in a database using sqlobject.
@@ -407,13 +527,18 @@ class simulate:
         
         DO.Site.sqlmeta.table = name
         DO.Edge.sqlmeta.table = name+'e'
+        if self.backend.lower()=='sqlite':
+            DO.Site.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
+            DO.Site._connection.commit()
+            DO.Edge.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
+            DO.Edge._connection.commit()
         #adding columns to sites table
         try: #Only add if they don't exist. On Batch runs this will catch the add error
             DO.Site.sqlmeta.addColumn(StringCol('incidence'))
             DO.Site.sqlmeta.addColumn(StringCol('arrivals'))
         except: pass
         if self.g.site_list[0].values:#add columns for 'values' variables
-            for i in range(len(self.g.site_list[0].values)):
+            for i in xrange(len(self.g.site_list[0].values)):
                 try:
                     DO.Site.sqlmeta.addColumn(StringCol('values%s'%i))
                 except: pass
@@ -425,7 +550,7 @@ class simulate:
         DO.Site.createTable()
         DO.Edge.createTable()
         #saving pickle of adjacency matrix
-        matname = 'adj_'+self.modelName.split('/')[-1] # table
+        matname = 'adj_'+self.modelName # table
         adjfile = open(matname,'w')
         cPickle.dump(self.g.getConnMatrix(),adjfile)
         adjfile.close()
@@ -460,11 +585,12 @@ class simulate:
                 for n,v in enumerate(site.vnames):
                     #print i[n],type( i[n])
                     dicin[v] = str(i[n])
-                DO.Site(**dicin)
+                    DO.Site(**dicin)
                 t += 1
-        DO.Site._connection.commit()
+        #DO.Site._connection.commit()
         #inserting data in edges table
         for e in self.g.edge_list:
+            rlist = []
             edicin={
             'source_code':e.source.geocode,
             'dest_code':e.dest.geocode}
@@ -473,9 +599,11 @@ class simulate:
                 edicin['time'] = t
                 edicin['ftheta'] = f
                 edicin['btheta'] = b
-                DO.Edge(**edicin)
+                rlist.append(edicin)
                 t += 1
-            DO.Edge._connection.commit()
+            insobj = DO.sqlbuilder.Insert(DO.Edge.sqlmeta.table,valueList=rlist)
+            DO.Edge._connection.queryAll(DO.Edge._connection.sqlrepr(insobj))
+            #DO.Edge._connection.commit()
     
     def parInsert(self):
         """
@@ -526,33 +654,52 @@ class simulate:
             sys.exit("commit successful")
     def outToDb(self,table):
         """
-        Insert simulation results on a mysql table
+        Insert simulation results on a mysql or SQLite table
         """
-        self.Say('Saving data on MySQL...')
+        if self.backend.lower() == "mysql":
+            self.Say('Saving data on MySQL...')
+        elif self.backend.lower() == "sqlite":
+            self.Say('Saving data on SQLite...')
         con=None
         try:
             table = table+'_'+self.now
             self.outtable = table
-            con = MySQLdb.connect(host=self.host, port=self.port,
+            if self.backend.lower() == "mysql":
+                con = MySQLdb.connect(host=self.host, port=self.port,
                                     user=self.usr,passwd=self.passw, db=self.db)
+            elif self.backend.lower() == "sqlite":
+                con = sqlite3.connect("Epigrass.db")
             # Define number of variables to be stored                         
             nvar = len(self.g.site_list[0].ts[0]) +4 #state variables,  plus coords, plus incidence, plus infected arrivals.
             str1 = '`%s` FLOAT(9),'*nvar #nvar variables in the table
+            str1lite = '%s REAL,'*nvar #nvar variables in the SQLite table
             varnames = ['lat','longit']+list(self.g.site_list[0].vnames)+['incidence']+['Arrivals']
-            str1 = str1[:len(str1)-1] % tuple(varnames) #insert variable names
+            str1 = str1[:len(str1)-1] % tuple(varnames) #insert variable names (MySQL)
+            str1lite = str1lite[:len(str1lite)-1] % tuple(varnames) #insert variable names (SQLITE)
             Cursor = con.cursor()
             str2 = """CREATE TABLE %s(
-            `geocode` INT( 9 ) DEFAULT '0' NOT NULL ,
-            `time` INT( 9 ) DEFAULT '' NOT NULL,
-            `name` varchar(128) DEFAULT '' NOT NULL,
+            `geocode` INT( 9 )  ,
+            `time` INT( 9 ) ,
+            `name` varchar(128) ,
+            """ % table
+            str2lite = """CREATE TABLE %s(
+            geocode INTEGER  ,
+            time INTEGER ,
+            name TEXT ,
             """ % table
             sql = str2+str1+');'
+            sqlite = str2lite+str1lite+');'
             
-            Cursor.execute(sql)
-            str3 = (nvar+3)*'%s,'
-            str3 = str3[:-1]+')'
+            if self.backend.lower() == "mysql":
+                Cursor.execute(sql)
+                str3 = (nvar+3)*'%s,'
+                str3 = str3[:-1]+')'
+            elif self.backend.lower() == "sqlite":
+                Cursor.execute(sqlite)
+                str3 = (nvar+3)*'?,'
+                str3 = str3[:-1]+')'
             sql2 = 'INSERT INTO %s' % table + ' VALUES('+str3
-            
+            nvalues = []
             for site in self.g.site_list:
                 geoc = site.geocode
                 lat = site.pos[0]
@@ -564,57 +711,54 @@ class simulate:
                 t = 0
                 for i in ts:
                     tstep = str(t)
-                    Cursor.execute(sql2,tuple([geoc,tstep,name]+[lat,long]+list(i)+[inc[t]]+[thist[t]]))
+                    nvalues.append(tuple([geoc,tstep,name]+[lat,long]+list(i)+[inc[t]]+[thist[t]]))
                     t += 1
-            
+            Cursor.executemany(sql2,nvalues)
             #Creating a table for edge data
             self.etable = etable = table+'e'
             esql = """CREATE TABLE %s(
-            `source_code` INT( 9 ) DEFAULT '0' NOT NULL ,
-            `dest_code` INT( 9 ) DEFAULT '0' NOT NULL ,
-            `time` INT( 9 ) DEFAULT '0' NOT NULL ,
-            `ftheta` FLOAT(9) DEFAULT '0' NOT NULL ,
-            `btheta` FLOAT(9) DEFAULT '0' NOT NULL);"""%etable
-            Cursor.execute(esql)
-            esql2 = 'INSERT INTO %s' % etable + ' VALUES(%s,%s,%s,%s,%s)'
+            `source_code` INT( 9 )  ,
+            `dest_code` INT( 9 )  ,
+            `time` INT( 9 ) ,
+            `ftheta` FLOAT(9) ,
+            `btheta` FLOAT(9) );"""%etable
+            esqlite = """CREATE TABLE %s(
+            source_code INTEGER  ,
+            dest_code INTEGER  ,
+            time INTEGER ,
+            ftheta REAL ,
+            btheta REAL );"""%etable
+            
+            if self.backend.lower() == "mysql":
+                Cursor.execute(esql)
+                esql2 = 'INSERT INTO %s' % etable + ' VALUES(%s,%s,%s,%s,%s)'
+            elif self.backend.lower() == "sqlite":
+                Cursor.execute(esqlite)
+                esql2 = 'INSERT INTO %s' % etable + ' VALUES(?,?,?,?,?)'
+            values = []
             for e in self.g.edge_list:
                 s = e.source.geocode
                 d = e.dest.geocode
                 t=0
                 for f,b in zip(e.ftheta,e.btheta):
-                    Cursor.execute(esql2,(s,d,t,f,b))
+                    values.append((s,d,t,f,b))
                     t += 1
+            Cursor.executemany(esql2,values)
             
             
         finally:
             if con:
                 con.close()
         #saving pickle of adjacency matrix
-        matname = 'adj_'+self.modelName.split('/')[-1] # table
-        adjfile = open(matname,'w')
+        matname = 'adj_'+self.modelName # table
+        fname = os.path.join(self.outdir,matname)
+        adjfile = open(fname,'w')
         cPickle.dump(self.g.getConnMatrix(),adjfile)
         adjfile.close()
         
-    def dumpData(self):
-        """
-        Dumps data as csv (comma-separated-values)
-        """
-##        pid = os.fork()
-##        if pid:
-##            return
-##        else:
-        curdir = os.getcwd()
-        if not self.outdir:
-            self.outdir = outdir = 'outdata-'+self.modelName.split('/')[-1]
-            try:
-                os.system('mkdir %s'%outdir)
-            except: pass #In case dir already exists (replicate runs)
-            os.chdir(outdir)
-        else:
-            os.system('mkdir %s'%self.outdir)
-            os.chdir(self.outdir)
-        codeslist = [str(i.geocode) for i in self.g.site_list]
+    def criaAdjMatrix(self):
         # saving the adjacency  matrix
+        codeslist = [str(i.geocode) for i in self.g.site_list]
         if not os.path.exists('adjmat.csv'): 
             print 'Saving the adjacency  matrix...'
             am = self.g.getConnMatrix()
@@ -625,9 +769,25 @@ class simulate:
                 amf.write(','.join(row)+'\n')
             amf.close()
             print 'Done!'
+    
+    def dumpData(self):
+        """
+        Dumps data as csv (comma-separated-values)
+        """
+##        pid = os.fork()
+##        if pid:
+##            return
+##        else:
+        self.Say("Starting simulation Analysis")
+        curdir = os.getcwd()
+        if not self.outdir == curdir:
+            os.chdir(outdir)
+        
+        codeslist = [str(i.geocode) for i in self.g.site_list]
+        self.criaAdjMatrix()
         #saving the shortest path matrices
         if not os.path.exists('spmat.csv'):
-            print 'Saving the shortest path matrices...'
+            self.Say('Calculating the shortest path matrices...')
             ap = self.g.getAllPairs()
             f = open('ap','w')
             cPickle.dump(ap,f)
@@ -679,12 +839,14 @@ class simulate:
         else:
             stf = codecs.open('epistats.csv','w',self.encoding)#create a new file
             stf.write('seed,name,size,infected_cities,spreadtime,median_survival,totvaccinated,totquarantined,seedcentrality,seedbetw,seedthidx,seeddeg,seedpop\n')
+##            stf.write('seed,name,size,infected_cities,spreadtime,median_survival,totvaccinated,totquarantined,seeddeg,seedpop\n')
         scent = str(seed.getCentrality())
         sbetw = str(seed.getBetweeness())
         sthidx = str(seed.getThetaindex())
         sdeg = str(seed.getDegree())
         spop = str(seed.totpop)
         sname =seed.sitename
+##        sstats = '%s,%s'%(sdeg,spop)
         sstats = '%s,%s,%s,%s,%s'%(scent,sbetw,sthidx,sdeg,spop)
         stf.write(str(self.seed[0][0])+','+sname+','+','.join(stats)+','+sstats+'\n')
         stf.close()
@@ -697,8 +859,9 @@ class simulate:
             sitef = codecs.open('sitestats.csv','a',self.encoding) #append to file
         else:
             sitef = codecs.open('sitestats.csv','w',self.encoding)
-        #sitef.write('site,centrality,degree,theta_index,betweeness,infection_time\n')
-        sitef.write('round,geocode,name,infection_time,degree,centrality,betweeness,theta_index,distance,seed,seedname\n')
+        
+        #sitef.write('round,geocode,name,infection_time,degree,centrality,betweeness,theta_index,distance,seed,seedname\n')
+        sitef.write('round,geocode,name,infection_time,degree,distance,seed,seedname\n')
         for s in self.g.site_list:
             degree = str(s.getDegree())
             central = str(s.getCentrality())
@@ -716,17 +879,18 @@ class simulate:
 ##            st = ','.join([str(i) for i in s.doStats()])
 ##            sitef.write(str(s.sitename)+','+st+it+'\n')
             sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+central+','+bet+','+thidx+','+distseed+','+seedgc+','+seedname+'\n')
+            sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+distseed+','+seedgc+','+seedname+'\n')
             
         os.chdir(curdir)
         
         print 'Done saving data!'
     
-    def saveModel(self,graph,fname):
+    def saveModel(self,fname):
         """
         Save the fully specified graph.
         """
         f = open(fname,'w')
-        cPickle.dump(graph,f)
+        cPickle.dump(self.g,f)
         f.close()
         
     def loadModel(self,fname):
@@ -873,3 +1037,5 @@ if __name__ == '__main__':
             rep = Rp.report(S)
             rep.Assemble(S.Rep)
 
+
+
