@@ -206,10 +206,12 @@ class siteobj:
         """
         if not self.isNode():
             return []
+        if self.neighbors:
+            return self.neighbors
         
         n=[[i.source,i.dest,i.length] for i in self.edges]
         neigh = {}
-        for j,i in enumerate(n):
+        for i in n:
             #print len(i[0]),i[0][0].sitename,self.sitename
             #if len(i[0])==1: print self.edges[j].sites,self.edges[j].source.sitename,self.edges[j].dest.sitename
             idx = i.index(self)
@@ -438,10 +440,10 @@ class popmodels:
         #beta=eval(self.values[2])
         
         Infectantes = Ig1+Ig2+Ig3+Ig4+Ic1+Ic2+Ic3+Ic4+0.5*(Is1+Is2+Is3+Is4)+theta
-        L1pos = beta*S1*(Infectantes/(N+npass))**alpha
-        L2pos = beta*S2*(Infectantes/(N+npass))**alpha
-        L3pos = beta*S3*(Infectantes/(N+npass))**alpha
-        L4pos = beta*S4*(Infectantes/(N+npass))**alpha
+        L1pos = float(beta)*S1*(Infectantes/(N+npass))**alpha
+        L2pos = float(beta)*S2*(Infectantes/(N+npass))**alpha
+        L3pos = float(beta)*S3*(Infectantes/(N+npass))**alpha
+        L4pos = float(beta)*S4*(Infectantes/(N+npass))**alpha
         
         ######################
         Lpos = L1pos+L2pos+L3pos+L4pos
@@ -501,7 +503,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         # Model
         Ipos = (1-r)*I + Lpos
@@ -531,7 +533,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -566,7 +568,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         # Model
         Ipos = (1-r)*I + Lpos
@@ -598,7 +600,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -634,7 +636,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         #print N,E,I,S,Lpos,theta
         Epos = (1-e)*E + Lpos
@@ -664,7 +666,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -699,7 +701,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         #print N,E,I,S,Lpos,theta
         Epos = (1-e)*E + Lpos
@@ -730,7 +732,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp) #poisson(Lpos_esp)
@@ -768,7 +770,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         # Model
         Ipos = (1-r)*I + Lpos
@@ -800,7 +802,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -837,7 +839,7 @@ class popmodels:
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
         
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         #print N,E,I,S,Lpos,theta
         Epos = (1-e)*E + Lpos
@@ -868,7 +870,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -905,8 +907,8 @@ class popmodels:
         N = self.parentSite.totpop
         R = N-E-I-S
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
-        Lpos2 = p*beta*R*((I+theta)/(N+npass))**alpha
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos2 = p*float(beta)*R*((I+theta)/(N+npass))**alpha
         self.parentSite.totalcases += Lpos+Lpos2 #update number of cases
         # Model
         Ipos = (1-r)*I + Lpos + Lpos2
@@ -941,8 +943,8 @@ class popmodels:
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
         R = N-E-I-S
         
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
-        Lpos2_esp = p*beta*R*((I+theta)/(N+npass))**alpha
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos2_esp = p*float(beta)*R*((I+theta)/(N+npass))**alpha
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -984,8 +986,8 @@ class popmodels:
         R = N-E-I-S
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
         
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
-        Lpos2 = p*beta*R*((I+theta)/(N+npass))**alpha
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos2 = p*float(beta)*R*((I+theta)/(N+npass))**alpha
         self.parentSite.totalcases += Lpos+Lpos2 #update number of cases
         # Model
         Epos = (1-e)*E + Lpos + Lpos2
@@ -1022,8 +1024,8 @@ class popmodels:
         
         R = N-E-I-S
         
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
-        Lpos2_esp = p*beta*R*((I+theta)/(N+npass))**alpha
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos2_esp = p*float(beta)*R*((I+theta)/(N+npass))**alpha
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -1065,7 +1067,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         self.parentSite.totalcases += Lpos #update number of cases
         # Model
         Ipos = (1-r)*I + Lpos
@@ -1097,7 +1099,7 @@ class popmodels:
             E,I,S = inits
         N = self.parentSite.totpop
         beta,alpha,e,r,delta,B,w,p = par#self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['B'],self.bp['w'],self.bp['p']
-        Lpos_esp = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
+        Lpos_esp = float(beta)*S*((I+theta)/(N+npass))**alpha #Number of new cases
         
         if dist == 'poisson':
             Lpos = poisson(Lpos_esp)
@@ -1291,6 +1293,7 @@ class graph:
             for w in G[v]:
                 vwLength = D[v] + G[v][w]
                 if w in D:
+                    #print vwLength
                     if vwLength < D[w]:
                         raise ValueError, \
       "Dijkstra: found better path to already-final vertex"
@@ -1409,7 +1412,7 @@ class graph:
         """
         Find a single shortest path from the given start node
         to the given end node.
-        The input has the same conventions as self.sijkstra().
+        The input has the same conventions as self.dijkstra().
         'G' is the graph's dictionary self.graphdict.
         'start' and 'end' are site objects.
         The output is a list of the vertices in order along
